﻿using System.Diagnostics;
using System.Net;
using System.Net.Sockets;

namespace AbstractTcpServer
{
    public abstract class TcpServer : ITcpServer
    {
        public int Port { get; }
        public IPAddress Address { get; }

        private TcpListener _server;
        private volatile bool _listening = true;

        public TcpServer(IPAddress address, int port)
        {
            Address = address;
            Port = port;
        }

        public void Start()
        {
            _server = new TcpListener(Address, Port);
            Trace.WriteLine("TcpListener object created");
            Trace.WriteLine("TCP server starting ...");
            _server.Start();
            Trace.WriteLine("TCP server started on " + Address + " port " + Port);

            while (_listening)
            {
                //if (!_server.Pending()) //checking for pending connections 
                //{
                //    Thread.Sleep(50);
                //    continue;
                //}
                TcpClient client = _server.AcceptTcpClient();
                EndPoint clientAddress = client.Client.RemoteEndPoint;
                Trace.WriteLine("Client connected: " + clientAddress);
                DoIt(client);
            }
        }

        public void Stop()
        {
            _listening = false;
            _server.Stop();
        }

        protected abstract void DoIt(TcpClient client);
    }
}