﻿using System.Diagnostics;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Threading.Tasks;
using AbstractTcpServer;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace UnitTestProject1
{
    [TestClass]
    public class UnitTest1
    {
        private const int Port = 12345;
        private const string Host = "127.0.0.1";
        private static ITcpServer _server;

        [ClassInitialize]
        public static void Start(TestContext context)
        {
            SetupTrace();

            IPAddress ipAddress = IPAddress.Parse(Host);
            _server = new EchoServer(ipAddress, Port);
            Task.Run(() => _server.Start());
        }

        private static void SetupTrace()
        {
            Trace.Listeners.Add(new ConsoleTraceListener());
            FileStream fs = new FileStream("c:/temp/echo.log", FileMode.Append);
            Trace.Listeners.Add(new TextWriterTraceListener(fs));
            Trace.AutoFlush = true;
        }

        [ClassCleanup]
        public static void Stop()
        {
            _server.Stop();
        }

        [TestMethod]
        public void TestServer()
        {
            Assert.AreEqual("hello world", GetResponse("hello world"));
        }

        private static string GetResponse(string request)
        {
            using (TcpClient client = new TcpClient(Host, Port))
            {
                NetworkStream stream = client.GetStream();
                StreamWriter streamWriter = new StreamWriter(stream);
                streamWriter.WriteLine(request);
                streamWriter.Flush();
                StreamReader streamReader = new StreamReader(stream);
                string response = streamReader.ReadLine();
                return response;
            }
        }
    }
}
